package main

import (
	"context"
	"flag"
	"io/ioutil"
	"log"
	"os"
	"os/signal"
	"syscall"

	"git.autistici.org/ai3/acmeserver"
	"git.autistici.org/ai3/go-common/serverutil"
	"gopkg.in/yaml.v2"
)

var (
	addr       = flag.String("addr", ":2780", "tcp `address` to listen on")
	configFile = flag.String("config", "/etc/acmeserver/config.yml", "configuration `file`")
)

type Config struct {
	ACME   *acmeserver.Config       `yaml:",inline"`
	Server *serverutil.ServerConfig `yaml:"http_server"`
}

func loadConfig(path string) (*Config, error) {
	// Read YAML config.
	data, err := ioutil.ReadFile(path)
	if err != nil {
		return nil, err
	}
	var config Config
	if err := yaml.Unmarshal(data, &config); err != nil {
		return nil, err
	}
	return &config, nil
}

func main() {
	log.SetFlags(0)
	flag.Parse()

	config, err := loadConfig(*configFile)
	if err != nil {
		log.Fatal(err)
	}

	acme, err := acmeserver.NewACME(config.ACME)
	if err != nil {
		log.Fatal(err)
	}

	m, err := acmeserver.NewManager(config.ACME, acme)
	if err != nil {
		log.Fatal(err)
	}

	// Start the acmeserver.Manager, and set up a SIGHUP handler to reload its config.
	if err := m.Start(context.Background()); err != nil {
		log.Fatal(err)
	}
	defer m.Stop()

	hupCh := make(chan os.Signal, 1)
	go func() {
		for {
			<-hupCh
			m.Reload()
		}
	}()
	signal.Notify(hupCh, syscall.SIGHUP)

	if err := serverutil.Serve(acme, config.Server, *addr); err != nil {
		log.Fatal(err)
	}
}
